

def estimate_confidence(i):  # NOSONAR
    """
    Estimate the confidence of an indicator
    :param i: csirtg_indicator.Indicator object
    :return: int
    """
    if i.is_hash:
        return 4

    if not getattr(i, 'tags'):
        return 0

    if i.is_email:
        if len(i.tags) > 1:
            return 4
        return 3

    if i.is_url:
        if len(i.tags) > 1 or 'phishing' in i.tags:
            return 4
        return 3

    if i.tags and len(i.tags) > 1:
        return 3

    if i.is_ip:
        if not i.tags:
            return 2

        if 'scanner' in i.tags or len(i.tags) > 1:
            return 4

    if i.tags and len(i.tags) > 1:
        return 3

    return 2



from datetime import datetime
from csirtg_dt import get as get_dt
from csirtg_indicator.utils import resolve_itype, ipv4_normalize
from urllib.parse import urlparse

from csirtg_indicator.utils.confidence import estimate_confidence


class PropertiesMixin(object):

    @staticmethod
    def _time_setter(v):
        if not v:
            return

        if isinstance(v, datetime):
            return v
        else:
            return get_dt(v).to('utc').datetime

    @property
    def indicator(self):
        return self.__indicator

    @indicator.setter
    def indicator(self, i):
        if not i:
            self._indicator = None
            return

        i = i.lower()
        self.itype = resolve_itype(i)
        self._indicator = i

        if self.itype == 'url':
            u = urlparse(self._indicator)
            self._indicator = u.geturl().rstrip('/').lower()

        if self.itype == 'ipv4':
            self._indicator = ipv4_normalize(self._indicator)

        if self.mask and (self.itype in ['ipv4', 'ipv6']):
            self._indicator = '{}/{}'.format(self._indicator, int(self.mask))
            self.mask = None

    @indicator.getter
    def indicator(self):
        return self._indicator

    @property
    def asn_desc(self):
        return self._asn_desc

    @asn_desc.setter
    def asn_desc(self, v):
        if not v:
            self._asn_desc = None
            return

        if isinstance(v, str):
            self._asn_desc = v
            return

        try:
            self._asn_desc = v.decode('utf-8')
            return
        except UnicodeDecodeError:
            pass

        try:
            self._asn_desc = v.decode('latin-8')
            return
        except UnicodeDecodeError as e:
            raise e

    @asn_desc.getter
    def asn_desc(self):
        return self._asn_desc

    @property
    def confidence(self):
        return self._confidence

    @property
    def reported_at(self):
        return self._reported_at

    @reported_at.getter
    def reported_at(self):
        return self._reported_at

    @reported_at.setter
    def reported_at(self, v):
        self._reported_at = self._time_setter(v)

    @property
    def last_at(self):
        return self._last_at

    @last_at.getter
    def last_at(self):
        return self._last_at

    @last_at.setter
    def last_at(self, v):
        self._last_at = self._time_setter(v)

    @property
    def first_at(self):
        return self._first_at

    @first_at.getter
    def first_at(self):
        return self._first_at

    @first_at.setter
    def first_at(self, v):
        self._first_at = self._time_setter(v)

    @confidence.setter
    def confidence(self, v):
        if v is not None:
            self._confidence = float(v)
        else:
            self._confidence = None

    @confidence.getter
    def confidence(self):
        if self._confidence is None:
            return estimate_confidence(self)
        return self._confidence

    @property
    def count(self):
        return self._count

    @count.setter
    def count(self, v):
        self._count = int(v)

    @count.getter
    def count(self):
        return self._count

    def get(self, v, default=None):
        v1 = getattr(self, v)
        if v1:
            return v1
        return default



import pytricia
from csirtg_indicator.utils.ip import is_ipv6
from csirtg_indicator.constants.networks import V6_RESERVED

PERM_WHITELIST = [
    'FF01:0:0:0:0:0:0:1',
    'FF01:0:0:0:0:0:0:2',
]


def process(data, whitelist=[]):
    wl = pytricia.PyTricia()

    [wl.insert(x, True) for x in PERM_WHITELIST]

    [wl.insert(str(y['indicator']), True) for y in whitelist]

    for i in data:
        if 'whitelist' in set(i['tags']):
            continue

        if not is_ipv6(i['indicator']):
            continue

        if i['indicator'] in V6_RESERVED:
            continue

        if str(i['indicator']) not in wl:
            yield i




import logging
ENABLED = True

try:
    from csirtg_geo import get

except (ImportError, FileNotFoundError) as e:
    ENABLED = False

logger = logging.getLogger(__name__)


class Geo(object):
    def geo_resolve(self):
        if not ENABLED:
            logger.error('maxmind data/libraries not installed')
            return

        rv = get(self.indicator)
        if not rv and self.rdata:
            rv = get(self.rdata[0])

        if not rv:
            return

        for k, v in rv.items():
            setattr(self, k, v)






class Farsight(object):

    def farsight(self):
        if self.itype != 'ipv4':
            raise TypeError('%s is not supported' % self.itype)

        try:
            from csirtg_dnsdb.client import Client
        except ImportError:
            print('')
            print('The csirtg function requires the csirtg_dnsdb client')
            print('https://github.com/csirtgadgets/dnsdb-py')
            print('$ pip install csirtg_dnsdb')
            print('$ export FARSIGHT_TOKEN=1234...')
            print('')
            raise SystemExit

        return Client().search(self.indicator)





class Cif(object):

    def cif(self):
        try:
            from cifsdk.client.http import HTTP as Client
        except ImportError:
            print('')
            print('The cif function requires the cifsdk>=4.0')
            print('$ pip install https://github.com/csirtgadgets/'
                  'verbose-robot-sdk-py/archive/master.zip')
            print('$ export CIF_TOKEN=1234...')
            print('')
            raise SystemExit

        return Client().search({'q': self.indicator, 'limit': 25})



import json
import textwrap
from argparse import ArgumentParser, RawDescriptionHelpFormatter
from datetime import datetime
from base64 import b64encode
import logging
import uuid
import copy
import arrow

from csirtg_indicator.constants import V4_RESERVED, VERSION, \
    FIELDS, FIELDS_TIME, LOG_FORMAT, VERSION, GEO, PEERS, FQDN, BASESTRING


from .wrappers.formatter import FormatterMixin
from .wrappers.itypes import ItypesMixin
from .wrappers.properties import PropertiesMixin
from .wrappers.spamhaus import Spamhaus as SpamhausMixin
from .wrappers.farsight import Farsight as FarsightMixin
from .wrappers.csirtg import Csirtg as CsirtgMixin
from .wrappers.cif import Cif as CifMixin
from .wrappers.geo import Geo as GeoMixin
from .wrappers.peers import Peers as PeersMixin
from .wrappers.dns import DNS as DNSMixin


class Indicator(PropertiesMixin, FormatterMixin,
                ItypesMixin, SpamhausMixin, FarsightMixin, CsirtgMixin,
                DNSMixin, CifMixin, GeoMixin, PeersMixin):

    def __init__(self, indicator=None, **kwargs):
        self.version = VERSION

        self._init_fields(**kwargs)

        if indicator:
            self.indicator = indicator

        # geo, fqdn, peers
        self._init_metadata(**kwargs)

    def _init_metadata(self, **kwargs):
        self.resolve_geo = kwargs.get('resolve_geo', GEO)
        self.resolve_peers = kwargs.get('resolve_peers', PEERS)
        self.resolve_fqdn = kwargs.get('resolve_fqdn', FQDN)

        if self.resolve_fqdn:
            self.fqdn_resolve()

        if self.resolve_geo:
            self.geo_resolve()

        if self.resolve_peers:
            self.peers_resolve()

    def _init_fields(self, **kwargs):
        for k in FIELDS:
            # handle these at the end
            if k in ['indicator', 'confidence', 'count']:
                setattr(self, f"_{k}", None)
                continue

            if kwargs.get(k) is None:
                v = None

                setattr(self, k, v)
                continue

            # set this at the end
            if k in FIELDS_TIME:
                continue

            if isinstance(kwargs[k], BASESTRING):
                kwargs[k] = kwargs[k].lower()
                if k in ['tags', 'peers', 'ptags', 'upstream', 'downstream']:
                    kwargs[k] = kwargs[k].split(',')

            setattr(self, k, kwargs[k])

        for k in FIELDS_TIME:
            setattr(self, k, kwargs.get(k, None))

        self.confidence = kwargs.get('confidence', None)
        self.count = kwargs.get('count', 1)
        self.uuid = kwargs.get('uuid', str(uuid.uuid4()))
        self.group = kwargs.get('group', 'everyone')
        self.tlp = kwargs.get('tlp', 'AMBER')
        self.asn_desc = None

        # incoming id
        self.iid = kwargs.get('iid')

        # outgoing id (if any)
        self.oid = kwargs.get('oid')

        if not self.reported_at:
            self.reported_at = arrow.utcnow().datetime

    def copy(self, **kwargs):
        try:
            i = Indicator(**copy.deepcopy(self.__dict__()))

            for k in kwargs:
                setattr(i, k, kwargs[k])

            i.uuid = str(uuid.uuid4())
            if not i.tags:
                i.tags = []

            if not isinstance(i.tags, list):
                i.tags = [i.tags]

            if not kwargs.get('last_at'):
                setattr(i, 'last_at', arrow.utcnow())

        except TypeError:
            i = None

        return i

    def __dict__(self):
        s = str(self)
        return json.loads(s)

    def _format_fields(self):
        i = {}
        for k in FIELDS:

            v = getattr(self, k)
            if not v and k == 'confidence':
                i[k] = 0.0

                continue

            if not v:
                continue

            if k == 'confidence':
                v = float(v)

            elif k in FIELDS_TIME and isinstance(v, datetime):
                v = v.strftime("%Y-%m-%dT%H:%M:%S.%fZ")

            elif k == 'message':
                v = b64encode(v.encode('utf-8')).decode('utf-8')

            else:
                if isinstance(v, BASESTRING):
                    v = v.lower()

            i[k] = v

        return i

    # https://stackoverflow.com/questions/48991911/how-to-write-a-custom-json-decoder-for-a-complex-object
    def __repr__(self):
        i = self._format_fields()

        sort_keys = False
        indent = None
        if logging.getLogger('').getEffectiveLevel() == logging.DEBUG:
            sort_keys = True
            indent = 4

        return json.dumps(i, indent=indent, sort_keys=sort_keys,
                          separators=(',', ': '))

    def __eq__(self, other):
        return self.__dict__() == other.__dict__()


def main():
    p = ArgumentParser(
        description=textwrap.dedent('''\
             Env Variables:
                CSIRTG_INDICATOR_TLP
                CSIRTG_INDICATOR_GROUP

            example usage:
                $ csirtg-indicator -d
            '''),
        formatter_class=RawDescriptionHelpFormatter,
        prog='csirtg-indicator'
    )

    p.add_argument('-d', '--debug', dest='debug', action="store_true")
    p.add_argument('-V', '--version', action='version', version=VERSION)

    p.add_argument('--group', help="specify group")
    p.add_argument('--indicator', '-i', help="specify indicator")
    p.add_argument('--tlp', help='specify tlp', default='green')
    p.add_argument('--tags', help='specify tags')

    args = p.parse_args()

    loglevel = logging.getLevelName('INFO')

    if args.debug:
        loglevel = logging.DEBUG

    console = logging.StreamHandler()
    logging.getLogger('').setLevel(loglevel)
    console.setFormatter(logging.Formatter(LOG_FORMAT))
    logging.getLogger('').addHandler(console)

    i = Indicator(indicator=args.indicator, tlp=args.tlp, tags=args.tags)
    i.geo_resolve()

    print(i)


if __name__ == '__main__':
    main()



import arrow
from csirtg_spamhaus import get


class Spamhaus(object):

    def spamhaus(self):

        if self.itype not in ['ipv4', 'ipv6']:
            return

        rv = get(self.indicator)
        if not rv:
            return

        confidence = 4
        if self.is_ip:
            ref = f"http://www.spamhaus.org/query/bl?ip={self.indicator}"

        else:
            if ' legit ' in rv['description']:
                confidence = 1

            ref = f"http://www.spamhaus.org/query/dbl?domain={self.indicator}"

        i = self.copy(**{
            'tags': rv['tags'],
            'description': rv['description'],
            'confidence': confidence,
            'provider': 'spamhaus.org',
            'reference': ref,
            'reference_tlp': 'white',
            'last_at': arrow.utcnow(),
        })

        if self.resolve_geo:
            i.geo_resolve()

        return i



from csirtg_indicator import Indicator


def test_confidence():
    i = Indicator('128.205.1.1', tags='scanner')
    assert i.confidence == 4

    i = Indicator('128.205.1.1')
    assert i.confidence == 0

    i = Indicator('http://go0gle.com/1.html', tags='phishing')
    assert i.confidence == 4



from csirtg_peers import get


class Peers(object):

    def peers_resolve(self):
        if self.is_private or not self.itype == 'ipv4':
            return

        self.peers = list(get(self.indicator))



import socket
import dns.resolver
from dns.resolver import NoAnswer, NXDOMAIN, NoNameservers, Timeout
from dns.name import EmptyLabel

from urllib.parse import urlparse

TIMEOUT = 5


def resolve_fqdn(host):
    if not host:
        return

    try:
        host = socket.gethostbyname(host)
        return host
    except Exception as e:
        return


def resolve_url(url):
    u = urlparse(url)
    return u.hostname


def resolve_ns(data, t='A', timeout=TIMEOUT, nameserver=None):
    resolver = dns.resolver.Resolver()
    resolver.timeout = timeout
    resolver.lifetime = timeout
    resolver.search = []

    if nameserver:
        resolver.nameservers = [nameserver]

    try:
        answers = resolver.query(data, t)

    except (NoAnswer, NXDOMAIN, EmptyLabel, NoNameservers, Timeout) as e:
        e = str(e)
        if e.startswith('The DNS operation timed out after'):
            return

        if 'The DNS response does not contain' in e or \
                'None of DNS query names exist' in e:
            return

        if 'failed to answer' in e:
            return

        raise

    resp = []
    for rdata in answers:
        resp.append(rdata)

    return resp




class Csirtg(object):
    def csirtg(self):
        try:
            from csirtgsdk.client import Client
            from csirtgsdk.search import Search
        except ImportError:
            print('')
            print('The csirtg function requires the csirtgsdk')
            print('$ pip install csirtgsdk')
            print('$ export CSIRTG_TOKEN=1234...')
            print('')
            raise SystemExit

        return Search(Client()).search(self.indicator, limit=5)



import os
from setuptools import setup, find_packages
import versioneer
import sys

# https://www.pydanny.com/python-dot-py-tricks.html
if sys.argv[-1] == 'test':
    test_requirements = [
        'pytest',
        'coverage',
        'pytest_cov',
    ]

    try:
        modules = map(__import__, test_requirements)

    except ImportError as e:
        err_msg = e.message.replace("No module named ", "")
        msg = "%s is not installed. Install your test requirements." % err_msg
        raise ImportError(msg)

    r = os.system('py.test test -sv --cov=csirtg_indicator '
                  '--cov-fail-under=65 --pep8')

    if r == 0:
        sys.exit()
    else:
        raise RuntimeError('tests failed')

setup(
    name="csirtg-indicator",
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    description="CSIRTG Indicator v3 Framework",
    long_description="",
    url="https://github.com/csirtgadgets/csirtg-indicator-v3",
    license='MPL2',
    classifiers=[
               "Programming Language :: Python",
               ],
    keywords=['security'],
    author="Wes Young",
    author_email="wes@csirtgadgets.com",
    packages=find_packages(),
    install_requires=[
        'csirtg-dt',
        'csirtg-re',
        'csirtg-peers',
        'csirtg-spamhaus',
        'csirtg-geo',
        'pytricia'
    ],
    scripts=[],
    entry_points={
        'console_scripts': [
            'csirtg-indicator=csirtg_indicator:main'
        ]
    },
)



from .fqdn import process as process_fqdn
from .ipv4 import process as process_ipv4
from .ipv6 import process as process_ipv6


def process(data, whitelist, itype=None):
    if itype == 'ipv4':
        yield process_ipv4(data, whitelist)

    elif itype == 'ipv6':
        yield process_ipv6(data, whitelist)

    elif itype == 'fqdn':
        yield process_ipv6(data, whitelist)

    else:
        whitelist = set(whitelist)

        for i in data:
            if 'whitelist' in set(i['tags']):
                continue

            if i['indicator'] in whitelist:
                continue

            yield i


def aggregate(data, field='indicator', sort='confidence',
              sort_secondary='reported_at'):
    x = set()
    rv = []
    for d in sorted(data, key=lambda x: x[sort], reverse=True):
        if d[field] not in x:
            x.add(d[field])
            rv.append(d)

    rv = sorted(rv, key=lambda x: x[sort_secondary], reverse=True)
    return rv



import os

from csirtg_re import *
from .fields import FIELDS, FIELDS_TIME
from .networks import V4_RESERVED, V6_RESERVED

VERSION = '3.0a0'

LOG_FORMAT = '%(asctime)s - %(levelname)s - %(name)s[%(lineno)s] - %(message)s'

COLUMNS = ['tlp', 'group', 'reported_at', 'indicator', 'risk', 'asn', 'cc',
           'first_at', 'last_at', 'count', 'tags', 'description', 'confidence',
           'provider', 'reference']

MAX_FIELD_SIZE = 30

BASESTRING = (str, bytes)

GEO = False
if os.getenv('CSIRTG_INDICATOR_GEO', '') == '1':
    GEO = True

PEERS = False
if os.getenv('CSIRTG_INDICATOR_PEERS', '') == '1':
    PEERS = True

FQDN = False
if os.getenv('CSIRTG_INDICATOR_FQDN', '') == '1':
    FQDN = True
